/*
 * IRC - Internet Relay Chat, ircd/m_sethost.c
 * Copyright (C) 1990 Jarkko Oikarinen and
 *                    University of Oulu, Computing Center
 *
 * See file AUTHORS in IRC package for additional names of
 * the programmers.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: m_sethost.c,v 1.5.2.1 2003/11/23 19:00:00 styx Exp $
 */

/*
 * m_functions execute protocol messages on this server:
 *
 *    cptr    is always NON-NULL, pointing to a *LOCAL* client
 *            structure (with an open socket connected!). This
 *            identifies the physical socket where the message
 *            originated (or which caused the m_function to be
 *            executed--some m_functions may call others...).
 *
 *    sptr    is the source of the message, defined by the
 *            prefix part of the message if present. If not
 *            or prefix not found, then sptr==cptr.
 *
 *            (!IsServer(cptr)) => (cptr == sptr), because
 *            prefixes are taken *only* from servers...
 *
 *            (IsServer(cptr))
 *                    (sptr == cptr) => the message didn't
 *                    have the prefix.
 *
 *                    (sptr != cptr && IsServer(sptr) means
 *                    the prefix specified servername. (?)
 *
 *                    (sptr != cptr && !IsServer(sptr) means
 *                    that message originated from a remote
 *                    user (not local).
 *
 *            combining
 *
 *            (!IsServer(sptr)) means that, sptr can safely
 *            taken as defining the target structure of the
 *            message in this server.
 *
 *    *Always* true (if 'parse' and others are working correct):
 *
 *    1)      sptr->from == cptr  (note: cptr->from == cptr)
 *
 *    2)      MyConnect(sptr) <=> sptr == cptr (e.g. sptr
 *            *cannot* be a local connection, unless it's
 *            actually cptr!). [MyConnect(x) should probably
 *            be defined as (x == x->from) --msa ]
 *
 *    parc    number of variable parameter strings (if zero,
 *            parv is allowed to be NULL)
 *
 *    parv    a NULL terminated list of parameter pointers,
 *
 *                    parv[0], sender (prefix string), if not present
 *                            this points to an empty string.
 *                    parv[1]...parv[parc-1]
 *                            pointers to additional parameters
 *                    parv[parc] == NULL, *always*
 *
 *            note:   it is guaranteed that parv[0]..parv[parc-1] are all
 *                    non-NULL pointers.
 */
#include "config.h"

#include "client.h"

#include "hash.h"
#include "ircd.h"
#include "msg.h"
#include "numnicks.h"
#include "send.h"

#include "ircd_reply.h"
#include "ircd_string.h"
#include "ircd_features.h"
#include "msgq.h"
#include "numeric.h"
#include "s_user.h"
#include "s_debug.h"
#include "struct.h"

#include <assert.h>
#include <stdlib.h>

/*
 * m_sethost par StyX
 *
 */
int m_sethost(struct Client* cptr, struct Client* sptr, int parc, char* parv[])
{
	struct Flags setflags;
	struct User*     user = cli_user(sptr);

	setflags = cli_flags(sptr);

	if (IsVhost(sptr)) {
		if (parc > 1) {
			if (!IsOper(sptr)) {
				send_reply(sptr, ERR_NOPRIVILEGES);
				return 0;
			}
			if (!is_hostmask(parv[1])) {
				sendcmdto_one(&me, CMD_NOTICE, sptr, "%C :Ce host est invalide (%s)", sptr, parv[1]);
				return 0;
			}
			/* On remplace le vrai host par la vhost sur user->host et user->realhost */
			ircd_strncpy(user->host, parv[1], HOSTLEN);
			ircd_strncpy(user->host, hostcrypt(user->host), HOSTLEN);
			ircd_strncpy(user->realhost, parv[1], HOSTLEN);
			sendcmdto_one(&me, CMD_NOTICE, sptr, "%C :Votre host est maintenant %s", sptr, user->realhost);
			sendto_opmask_butone(0, SNO_OLDSNO, "%s (%s@%s) change son host pour %s", parv[0], cli_user(sptr)->username, user->basehost, user->realhost);
		}
		else {
			ircd_strncpy(user->host, hostcrypt(user->basehost), HOSTLEN);
			ircd_strncpy(user->realhost, user->basehost, HOSTLEN);
			sendcmdto_one(&me, CMD_NOTICE, sptr, "%C :Retour  %s", sptr, user->basehost);
			sendcmdto_one(&me, CMD_NOTICE, sptr, "%C :Votre host n'est maintenant plus masqu", sptr);
			ClearVhost(sptr);
			sendto_opmask_butone(0, SNO_OLDSNO, "%s (%s@%s) revient  son vrai host", parv[0], cli_user(sptr)->username, user->basehost);
		}
	}
	else {	
		if (parc > 1) {
			if (!IsOper(sptr)) {
				send_reply(sptr, ERR_NOPRIVILEGES);
				return 0;
			}
			if (!is_hostmask(parv[1])) {
				sendcmdto_one(&me, CMD_NOTICE, sptr, "%C :Ce host est invalide (%s)", sptr, parv[1]);
				return 0;
			}
			/* On remplace le vrai host par la vhost sur user->host et user->realhost */
			ircd_strncpy(user->host, parv[1], HOSTLEN);
			ircd_strncpy(user->host, hostcrypt(user->host), HOSTLEN);
			ircd_strncpy(user->realhost, parv[1], HOSTLEN);
			sendcmdto_one(&me, CMD_NOTICE, sptr, "%C :Votre host est maintenant %s", sptr, user->realhost);
			sendcmdto_one(&me, CMD_NOTICE, sptr, "%C :Tapez /sethost pour revenir  votre vrai host (%s)", sptr, user->basehost);
			SetVhost(sptr);
			sendto_opmask_butone(0, SNO_OLDSNO, "%s (%s@%s) change son host pour %s", parv[0], cli_user(sptr)->username, user->basehost, user->realhost);
		}
		else {
			if (!IsOper(sptr)) {
				send_reply(sptr, ERR_NOPRIVILEGES);
				return 0;
			}
			return need_more_params(sptr, "SETHOST");
		}
	}
	send_umode_out(cptr, sptr, &setflags, 0);
}
